=begin
#Payments Orchestrator

#\"REST API for Payments Orchestrator\"

The version of the OpenAPI document: 1.6.4-SNAPSHOT
Contact: test@yapstone.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.1.0-SNAPSHOT

=end

require 'date'

module Yapstone
  # The payment method and any additional PII Yapstone may need to collect for risk purposes in order to process this payment. 
  class PaymentDetails
    # If your payment is for multiple line items, detail them here. The sum of your line item amounts must equal the amount given at the root of the request body. 
    attr_accessor :line_items

    # Used to list the authorization type. If it is PRIORAUTH this is a pre-authorization that places a hold on your customer's credit card for a specified dollar amount based on a projected sale amount. FINALAUTH is when you have completed all work checking the card and the customer and you are ready to actually withdraw the payment. 
    attr_accessor :authorization_type

    # The descriptor that will be shown on the customer's bank statement after the transaction has been authorized. 
    attr_accessor :custom_soft_descriptor

    attr_accessor :installment_plan

    attr_accessor :amount

    attr_accessor :payment_instrument

    # This indicates whether it is true that the payment method a customer wants to use belongs to the customer or not.
    attr_accessor :instrument_owner

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'line_items' => :'lineItems',
        :'authorization_type' => :'authorizationType',
        :'custom_soft_descriptor' => :'customSoftDescriptor',
        :'installment_plan' => :'installmentPlan',
        :'amount' => :'amount',
        :'payment_instrument' => :'paymentInstrument',
        :'instrument_owner' => :'instrumentOwner'
      }
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'line_items' => :'Array<OfferingDetails>',
        :'authorization_type' => :'String',
        :'custom_soft_descriptor' => :'String',
        :'installment_plan' => :'InstallmentPlan',
        :'amount' => :'Amount',
        :'payment_instrument' => :'PaymentInstrument',
        :'instrument_owner' => :'Boolean'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      if (!attributes.is_a?(Hash))
        fail ArgumentError, "The input argument (attributes) must be a hash in `Yapstone::PaymentDetails` initialize method"
      end

      # check to see if the attribute exists and convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h|
        if (!self.class.attribute_map.key?(k.to_sym))
          fail ArgumentError, "`#{k}` is not a valid attribute in `Yapstone::PaymentDetails`. Please check the name to make sure it's valid. List of attributes: " + self.class.attribute_map.keys.inspect
        end
        h[k.to_sym] = v
      }

      if attributes.key?(:'line_items')
        if (value = attributes[:'line_items']).is_a?(Array)
          self.line_items = value
        end
      end

      if attributes.key?(:'authorization_type')
        self.authorization_type = attributes[:'authorization_type']
      else
        self.authorization_type = 'FINALAUTH'
      end

      if attributes.key?(:'custom_soft_descriptor')
        self.custom_soft_descriptor = attributes[:'custom_soft_descriptor']
      end

      if attributes.key?(:'installment_plan')
        self.installment_plan = attributes[:'installment_plan']
      end

      if attributes.key?(:'amount')
        self.amount = attributes[:'amount']
      end

      if attributes.key?(:'payment_instrument')
        self.payment_instrument = attributes[:'payment_instrument']
      end

      if attributes.key?(:'instrument_owner')
        self.instrument_owner = attributes[:'instrument_owner']
      else
        self.instrument_owner = true
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if !@custom_soft_descriptor.nil? && @custom_soft_descriptor.to_s.length > 36
        invalid_properties.push('invalid value for "custom_soft_descriptor", the character length must be smaller than or equal to 36.')
      end

      if !@custom_soft_descriptor.nil? && @custom_soft_descriptor.to_s.length < 0
        invalid_properties.push('invalid value for "custom_soft_descriptor", the character length must be great than or equal to 0.')
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      authorization_type_validator = EnumAttributeValidator.new('String', ["FINALAUTH", "PRIORAUTH", ""])
      return false unless authorization_type_validator.valid?(@authorization_type)
      return false if !@custom_soft_descriptor.nil? && @custom_soft_descriptor.to_s.length > 36
      return false if !@custom_soft_descriptor.nil? && @custom_soft_descriptor.to_s.length < 0
      true
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] authorization_type Object to be assigned
    def authorization_type=(authorization_type)
      validator = EnumAttributeValidator.new('String', ["FINALAUTH", "PRIORAUTH", ""])
      unless validator.valid?(authorization_type)
        fail ArgumentError, "invalid value for \"authorization_type\", must be one of #{validator.allowable_values}."
      end
      @authorization_type = authorization_type
    end

    # Custom attribute writer method with validation
    # @param [Object] custom_soft_descriptor Value to be assigned
    def custom_soft_descriptor=(custom_soft_descriptor)
      if !custom_soft_descriptor.nil? && custom_soft_descriptor.to_s.length > 36
        fail ArgumentError, 'invalid value for "custom_soft_descriptor", the character length must be smaller than or equal to 36.'
      end

      if !custom_soft_descriptor.nil? && custom_soft_descriptor.to_s.length < 0
        fail ArgumentError, 'invalid value for "custom_soft_descriptor", the character length must be great than or equal to 0.'
      end

      @custom_soft_descriptor = custom_soft_descriptor
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          line_items == o.line_items &&
          authorization_type == o.authorization_type &&
          custom_soft_descriptor == o.custom_soft_descriptor &&
          installment_plan == o.installment_plan &&
          amount == o.amount &&
          payment_instrument == o.payment_instrument &&
          instrument_owner == o.instrument_owner
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Integer] Hash code
    def hash
      [line_items, authorization_type, custom_soft_descriptor, installment_plan, amount, payment_instrument, instrument_owner].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def self.build_from_hash(attributes)
      new.build_from_hash(attributes)
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.openapi_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :Boolean
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        Yapstone.const_get(type).build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
